#####################
####  R codes of the 3rd lecture of the applied spatial econometrics course
####  Analyzing Spatial Autocorrelation with R
####  T. Laurent (17/02/2016) 

###################################
#####  Slide p.6
###################################

install.packages(c("spdep","dismo","RColorBrewer","classInt","GISTools",
 "maptools","GeoXp","xtable"))

require("spdep")    # spatial econometrics library
require("dismo")    # google maps
require("RColorBrewer")     # the colors
require("classInt")   # transform continuous variable into discrete
require("GISTools")   # obtain some transparents colors
require("maptools")   # tools for cartography
require("GeoXp")      # spatial interactive exploratory analysis
require("xtable")     # tools for reporting

# reference of the article :
# http://www.colorado.edu/ibs/crs/workshops/R_1-11-2012/root/Harrison_1978.pdf
# import the data
require("maptools")
boston.tr <- readShapePoly(system.file("etc/shapes/boston_tracts.shp",
  package="spdep")[1], ID="poltract",
  proj4string=CRS(paste("+proj=longlat +datum=NAD27 +no_defs +ellps=clrk66",
  "+nadgrids=@conus,@alaska,@ntv2_0.gsb,@ntv1_can.dat")))

# the ID's of the tracts
ID <- row.names(boston.tr)
n.boston <- length(ID)

###################################
#####  Slide p.7
###################################

# variable of interest : median values of owner-occupied housing in USD 1000
spplot(boston.tr, "MEDV")
# explanatory variable :  
spplot(boston.tr, "NOX")


###################################
#####  Slide p.8,9
###################################

# vizualize the geographic situation of Boston
plotclr <- rev(add.alpha(brewer.pal(5,"RdBu"), 0.7))
# variable of interest
y <- boston.tr$MEDV
bk <- round(classIntervals(y, 5,"kmeans")$brks, digits=1)
ind <- findInterval(y, bk, all.inside=TRUE)
# we import a map
g <- gmap(boston.tr, type = "roadmap", zoom = 10)
# we identify the CRS of the map
projMer <- projection(g)
# we convert the data
boston.tr<- spTransform(boston.tr, CRS(projMer))
# we plot the information
plot(g)
plot(boston.tr, col=plotclr[ind], add=TRUE)
# the legend
decoup <- c(paste("<=",bk[2],sep=""),paste("]",bk[2],";",bk[3],"]",sep=""),
 paste("]",bk[3],";",bk[4],"]",sep=""),paste("]",bk[4],";",bk[5],"]",sep=""),paste(">",bk[5], sep=""))
legend("topleft", legend = decoup,
cex = 0.8,title="median values in USD 1000", fill=plotclr)

# how to get extra data
# https://worldmap.harvard.edu/maps/3948

###################################
#####  Slide p.10, 11
###################################

# linear model
boston.lm <- lm(log(MEDV) ~ CRIM + ZN + INDUS + CHAS + I(NOX^2) + I(RM^2) +
 AGE + log(DIS) + log(RAD) + TAX + PTRATIO + B + log(LSTAT), data=boston.tr@data)
summary(boston.lm)

# representation of the residuals
res.boston <- residuals(boston.lm)

# we cut the residuals into 5 classes
# mapping
bk <- round(classIntervals(res.boston, 5,"kmeans")$brks, digits=1)
ind <- findInterval(res.boston, bk, all.inside=TRUE)
plot(boston.tr, col=plotclr[ind], axes=T)
decoup <- c(paste("<=",bk[2],sep=""),paste("]",bk[2],";",bk[3],"]",sep=""),
 paste("]",bk[3],";",bk[4],"]",sep=""),paste("]",bk[4],";",bk[5],"]",sep=""),paste(">",bk[5], sep=""))
legend("topleft", legend = decoup,
cex = 0.8,title="residuals", fill=plotclr)


###################################
#####  Slide p.14
###################################

# we define the centroid of the tracts
coord <- coordinates(boston.tr)

# example: 
poly.ex <- boston.tr[c(1:7,9),]
coord.ex <- coord[c(1:7,9),]
plot(poly.ex)


 
###################################
#####  Slide p.18
###################################

poly.ex_nb.1 <- poly2nb(poly.ex)
# structure
str(poly.ex_nb.1)
# number of neighbours per observations
card(poly.ex_nb.1)
# number of observations
length(poly.ex_nb.1)
# symetric ?
is.symmetric.nb(poly.ex_nb.1)
# a summary
summary(poly.ex_nb.1)

 
###################################
#####  Slide p.19
###################################

# plot function
plot(poly.ex, border="grey", axes=TRUE, main="contiguity")
plot(poly.ex_nb.1, coord.ex, add=TRUE)

# distance
dlist <- nbdists(poly.ex_nb.1, coord.ex, longlat=FALSE)
dlist <- lapply(dlist, max)
hist(unlist(dlist))

# drop one neighbour
subset(poly.ex_nb.1, row.names(poly.ex)!="101")

###################################
#####  Slide p.20
###################################

# who are the neighbours of my neighbours ?
# the neighbours of the neighbours
lag.nb <- nblag(poly.ex_nb.3, 4)
op <- par(mfrow=c(1,4), oma=c(0,0,0,0), mar=c(0,0,0,0))
plot(poly.ex, border="grey", main="1st order")
plot(lag.nb[[1]], coord.ex, add=TRUE)
plot(poly.ex, border="grey", main="2nd order")
plot(lag.nb[[2]], coord.ex, add=TRUE)
plot(poly.ex, border="grey", main="3rd order")
plot(lag.nb[[3]], coord.ex, add=TRUE)
plot(poly.ex, border="grey", main="4th order")
plot(lag.nb[[4]], coord.ex, add=TRUE)
par(op)


###################################
#####  Slide p.21
###################################

poly.ex_nb.2 <- tri2nb(coord.ex)
# some caraterisics
is.symmetric.nb(poly.ex_nb.2)
summary(poly.ex_nb.2)
# plot function
plot(poly.ex, border="grey", axes=TRUE, main="Delaunay")
plot(poly.ex_nb.2, coord.ex, add=TRUE)


###################################
#####  Slide p.22
###################################

poly.ex_knn <- knearneigh(coord.ex, k=3, longlat = FALSE)
poly.ex_nb.3 <- knn2nb(poly.ex_knn)
# some caraterisics
is.symmetric.nb(poly.ex_nb.3)
summary(poly.ex_nb.3)
# plot function
plot(poly.ex, border="grey", axes=TRUE, main="k-nearest neighbours")
plot(poly.ex_nb.3, coord.ex, add=TRUE)

###################################
#####  Slide p.23
###################################

poly.ex_nb.4 <- dnearneigh(coord.ex, 0, 4000, longlat = FALSE)
# some caraterisics
is.symmetric.nb(poly.ex_nb.4)
summary(poly.ex_nb.4)
# plot function
plot(poly.ex, border="grey", axes=TRUE, main="distance between lower and upper bounds")
plot(poly.ex_nb.4, coord.ex, add=TRUE)

###################################
#####  Slide p.24
###################################

# summary of the neighbours types
op <- par(mfrow=c(2,2), oma=c(0,0,0,0), mar=c(0,0,1,0))
plot(poly.ex, border="grey", main="contiguity")
plot(poly.ex_nb.1, coord.ex, add=TRUE)
plot(poly.ex, border="grey", main="Delaunay")
plot(poly.ex_nb.2, coord.ex, add=TRUE)
plot(poly.ex, border="grey", main="k-nearest neighbours")
plot(poly.ex_nb.3, coord.ex, add=TRUE)
plot(poly.ex, border="grey", main="distance between lower and upper bounds")
plot(poly.ex_nb.4, coord.ex, add=TRUE)
par(op)


###################################
#####  Slide p.25
###################################

# contiguity + 1-nearest neighbour
poly.ex_nb.5 <- union.nb(poly.ex_nb.1, knn2nb(knearneigh(coord.ex, k=1, longlat = FALSE), row.names = row.names(poly.ex))) 
# plot function
plot(poly.ex, border="grey", axes=TRUE, main="contiguity + 1-nearest neighbour")
plot(poly.ex_nb.5, coord.ex, add=TRUE)

###################################
#####  Slide p.26
###################################

# distance + k-nearest
poly.ex_nb.6 <- intersect.nb(poly.ex_nb.3, poly.ex_nb.4) 
# plot function
plot(poly.ex, border="grey", axes=TRUE, main="k-nearest + distance")
plot(poly.ex_nb.6, coord.ex, add=TRUE)

###################################
#####  Slide p.27
###################################

x = GridTopology(c(2,2), c(1,1), c(4,4))
xy <- coordinates(x)
# rook
plot(as(x, "SpatialPolygons"), border="grey")
nb.rook <- cell2nb(4, 4, type="rook")
plot(nb.rook, xy, add=T, main="rook")
summary(nb.rook)
# queen
plot(as(x, "SpatialPolygons"), border="grey")
nb.queen <- cell2nb(4, 4, type="queen")
plot(nb.queen, xy, add=T, main="queen")
summary(nb.queen)

# more documentation :
vignette("nb", package="spdep")

###################################
#####  Slide p.29
###################################

# a. Binary matrix
# we give 1 to each neighbours  
poly.ex_listw.a <- nb2listw(poly.ex_nb.5, style="B")
str(poly.ex_listw.a)      # contains both neigbours list and weights lists

# If we consider a spatial laged variable Wx 
# this kind of weight are interesting because it gives the total number observed 
# on the neighbours : for example, we get here the number of neighbours whose border
# Charles river.
cbind(as.numeric(poly.ex@data$CHAS)-1,
lag(poly.ex_listw.a, as.numeric(poly.ex@data$CHAS)-1))

###################################
#####  Slide p.30
###################################
# b. row-normalized matrix
# economic motivation (Lesage and Pace, 2009)
poly.ex_listw.b <- nb2listw(poly.ex_nb.5, style="W")
summary(unlist(poly.ex_listw.b$weights))

# If we consider a spatial laged variable Wx 
# this kind of weight are interesting because it gives the average mean observed 
# on the neighbours : example with CRIM variable 
x <- poly.ex@data$CRIM
wx <- lag(poly.ex_listw.b, x)
plot(x, wx, xlab="variable CRIM", ylab="W x CRIM")
abline(h=mean(wx), col="grey", lty=2)
abline(v=mean(x), col="grey", lty=2)


###################################
#####  Slide p.31
###################################
dlist <- nbdists(poly.ex_nb.3, coord.ex)
dlist.c <- lapply(dlist, function(x) 1/x)
poly.ex_listw.c <- nb2listw(poly.ex_nb.3, glist=dlist.c, style="W")
summary(unlist(poly.ex_listw.c$weights))


###################################
#####  Slide p.32
###################################

pal <- c("white",brewer.pal(5, "Reds"))
# RSB quietening greys
#pal <- grey.colors(9, 1, 0.5, 2.2)
oopar <- par(mfrow=c(1,3), mar=c(1,1,3,1)+0.1)
z <- t(listw2mat(poly.ex_listw.a))
brks <- c(0,0,0.2,0.3,0.5,0.75,1)
nbr3 <- length(brks)-3
image(1:8, 1:8, z[,ncol(z):1], breaks=brks, col=pal,
 main="B style", axes=FALSE)
legend("bottomleft", legend=c("0","]0,0.2]","]0.2,0.3]","]0.3,0.5]","]0.5,0.75]","]0.75,1]"), fill=pal)
box()
z <- t(listw2mat(poly.ex_listw.b))
image(1:8, 1:8, z[,ncol(z):1], breaks=brks,  col=pal, 
main="W style", axes=FALSE)
box()
z <- t(listw2mat(poly.ex_listw.c))
image(1:8, 1:8, z[,ncol(z):1], breaks=brks, col=pal,
 main="IDW B style", axes=FALSE)
box()
par(oopar)
dev.null <- dev.off()


###################################
#####  Slide p.34
###################################
## Application to the boston full data
boston_nb.1 <- poly2nb(boston.tr)
boston_nb.2 <- dnearneigh(coord, 0, 2000, longlat = FALSE, row.names=ID)
boston_nb <- union.nb(boston_nb.1, boston_nb.2) 
# plot function
plot(boston.tr, border="grey", axes=TRUE, main="contiguity + 1-nearest neighbour")
plot(boston_nb, coord, add=TRUE)

###################################
#####  Slide p.35
###################################

# spatial exploratory analysis of the spatial weight matrix  
 histnbmap(boston.tr, boston_nb)
 barnbmap(boston.tr, boston_nb)
 neighbourmap(boston.tr, "LAT", boston_nb)
 
 ###################################
#####  Slide p.36
###################################
# we build the spatial weights 
dlist <- lapply(nbdists(boston_nb, coord), function(x) 1/x) 
boston_listw <- nb2listw(boston_nb, glist=dlist, style="W")
summary(unlist(boston_listw$weights))
 
# vizualisation of the matrix
pal <- c("white",brewer.pal(5, "Reds"))
# RSB quietening greys
#pal <- grey.colors(9, 1, 0.5, 2.2)
oopar <- par(oma=c(0,0,0,0), mar=c(1,1,3,1)+0.1)
z <- t(listw2mat(boston_listw))
brks <- c(0,0,0.2,0.3,0.5,0.75,1)
nbr3 <- length(brks)-3
image(1:n.boston, 1:n.boston, z[,ncol(z):1], breaks=brks, col=pal,
 main="Spatial Boston weight matrix", axes=FALSE)
box()
par(oopar)
 
 
###################################
#####  Slide p.37
###################################
# Some remarks
# the size of the matrix
boston_mat <- listw2mat(boston_listw)
object.size(boston_listw)
object.size(boston_mat)
boston_sparse <- as_dgRMatrix_listw(boston_listw)
object.size(boston_sparse)




###################################
#####  Slide p.39,40,41
###################################

uncorr_x <- rnorm(n.boston)
plotclr <- rev(brewer.pal(5,"Greys"))
# choice of rho 
dev.new(2)
par(mfrow=c(2,4), oma=c(0,0,0,0),mar=c(0,0,1,0))
dev.new(3)
par(mfrow=c(2,4), oma=c(0,0,0,0),mar=c(2,1.9,2.1,1))
for(rho in c(-0.1, -0.3, -0.6, -0.9, 0.1, 0.3, 0.6, 0.9))
{autocorr_x <- invIrW(boston_listw, rho) %*% uncorr_x 
 bk <- quantile(autocorr_x, seq(0,1,0.2))
 ind <- findInterval(autocorr_x, bk, all.inside=TRUE)
 dev.set(2)
 plot(boston.tr, col=plotclr[ind], border=add.alpha(plotclr[ind],0.3), main=paste("rho=",round(rho,1)))
 dev.set(3)
 plot(autocorr_x, lag(boston_listw, autocorr_x), xlab="random variable", 
  ylab="spatial lag",main=paste("rho=",round(rho,1)))
  lines(lowess(lag(boston_listw, autocorr_x)~autocorr_x), col="grey", lty=2)
}


###################################
#####  Slide p.44
###################################

res.unc.1 <- moran.test(uncorr_x, boston_listw, randomisation=TRUE)
res.unc.1a <- moran.test(autocorr_x, boston_listw, randomisation=TRUE)

res.unc.2 <- moran.test(uncorr_x, boston_listw, randomisation=FALSE)
res.unc.2a <-moran.test(autocorr_x, boston_listw, randomisation=FALSE)

res1 <- matrix("", ncol=5, nrow=4)
rownames(res1) <- c("rando uncorr", "rando autocorr", "norm uncorr", "norm autocorr")
colnames(res1) <- c("$I$", "$E(I)$", "$var(I)$", "st. deviate", "$p$-value")
res1[1, 1:3] <- format(res.unc.1$estimate, digits=2)
res1[1, 4] <- format(res.unc.1$statistic, digits=3)
res1[1, 5] <- format.pval(res.unc.1$p.value, digits=2, eps=1e-8)
res1[2, 1:3] <- format(res.unc.1a$estimate, digits=2)
res1[2, 4] <- format(res.unc.1a$statistic, digits=3)
res1[2, 5] <- format.pval(res.unc.1a$p.value, digits=2, eps=1e-8)
res1[3, 1:3] <- format(res.unc.2$estimate, digits=2)
res1[3, 4] <- format(res.unc.2$statistic, digits=3)
res1[3, 5] <- format.pval(res.unc.2$p.value, digits=2, eps=1e-8)
res1[4, 1:3] <- format(res.unc.2a$estimate, digits=2)
res1[4, 4] <- format(res.unc.2a$statistic, digits=3)
res1[4, 5] <- format.pval(res.unc.2a$p.value, digits=2, eps=1e-8)
print(xtable(res1, align=c("l", rep("r", 5))), floating=FALSE,
 sanitize.text.function = function(SANITIZE) SANITIZE)

###################################
#####  Slide p.45,46
###################################
# permutation test
res.unc <- moran.mc(uncorr_x, boston_listw, nsim=1000)
res.aut <- moran.mc(autocorr_x, boston_listw, nsim=1000)

# uncorrelated data
hist(res.unc$res,col="lightblue",freq=F,xlab="Simulated Moran I statistic",
main="Permutation test for Moran's I statistic",ylim=c(0,16))
dens.v<-density(res.unc$res)
lines(dens.v,col="red",lwd=1.5)
segments(res.unc$statistic,0,res.unc$statistic,max(dens.v$y), lwd = 2, col = "purple",lty=2)
legend("topleft",c("kernel density function estimates","Observed Moran's I statistic"),
lty=c(1,2),col=c("red","purple"),cex=0.8)
legend("topright", legend = c("Permutation : 999",paste("p-value :",res.unc$p.value)),cex=0.8)

# correlated data
hist(res.aut$res,col="lightblue",freq=F,xlab="Simulated Moran I statistic",
main="Permutation test for Moran's I statistic",ylim=c(0,16))
dens.v<-density(res.aut$res)
lines(dens.v,col="red",lwd=1.5)
segments(res.aut$statistic,0,res.aut$statistic,max(dens.v$y), lwd = 2, col = "purple", lty=2)
legend("topleft",c("kernel density function estimates","Observed Moran's I statistic"),
 lty=c(1,2), col=c("red","purple"), cex=0.8)
legend("topright", legend = c("Permutation : 999",paste("p-value :",res.aut$p.value)),cex=0.8)


###################################
#####  Slide p.47
###################################
lm.morantest(boston.lm, boston_listw)


###################################
#####  Slide p.48
###################################

box1<-matrix(rep(c(1,0,1,0,0,1,0,1),2),4,4)
set.seed(5)
box2<-matrix(sample(rep(0:1,8)),4,4)
box3<-matrix(c(rep(1,8),rep(0,8)),4,4)
#dev.new(width=10,height=5)
#dev.new()
op <- par(mfrow=c(1,3))
image(1:4,1:4,box1,col=c("black","white"),axes=FALSE,xlab="",ylab="",main="Alternating",ylim=c(-1,5))
segments(c(0.5,0.5,4.5,4.5),c(0.5,4.5,4.5,0.5),c(0.5,4.5,4.5,0.5),c(4.5,4.5,0.5,0.5))
text(2.5,0,"BB = 0",cex=1)
text(2.5,-0.25,"WW = 0",cex=1)
text(2.5,-0.5,"BW = 24",cex=1)
image(1:4,1:4,box2,col=c("black","white"),axes=FALSE,xlab="",ylab="",main="Random",ylim=c(-1,5))
segments(c(0.5,0.5,4.5,4.5),c(0.5,4.5,4.5,0.5),c(0.5,4.5,4.5,0.5),c(4.5,4.5,0.5,0.5))
text(2.5,0,"BB = 5",cex=1)
text(2.5,-0.25,"WW = 5",cex=1)
text(2.5,-0.5,"BW = 13.5",cex=1)
image(1:4,1:4,box3,col=c("black","white"),axes=FALSE,xlab="",ylab="",main="Clustered",ylim=c(-1,5))
segments(c(0.5,0.5,4.5,4.5),c(0.5,4.5,4.5,0.5),c(0.5,4.5,4.5,0.5),c(4.5,4.5,0.5,0.5))
text(2.5,0,"BB = 10",cex=1)
text(2.5,-0.25,"WW = 10",cex=1)
text(2.5,-0.5,"BW = 4",cex=1)
par(op)


###################################
#####  Slide p.49
###################################

library(xtable)
xtable(joincount.multi(factor(as.vector(box1), labels=c("B","W")), nb2listw(nb.rook, style="B")))
xtable(joincount.multi(factor(as.vector(box2), labels=c("B","W")), nb2listw(nb.rook, style="B")))
xtable(joincount.multi(factor(as.vector(box3), labels=c("B","W")), nb2listw(nb.rook, style="B")))

###################################
#####  Slide p.50
###################################
# binary rook matrix
res.BB<-joincount.mc(factor(as.vector(box1), labels=c("B","W")), nb2listw(nb.rook, style="B"), nsim=999)
dens.v <- density(res.BB[[1]]$res, bw=1)
hist(res.BB[[1]]$res,col="lightblue",breaks=12,freq=F,xlab="Simulated BB join count statistic",
 main="Permutation test for BB join count statistic",ylim=c(0,0.4))
lines(dens.v, col="red",lwd=1.5)
segments(res.BB[[1]]$statistic,0, res.BB[[1]]$statistic, max(dens.v$y), lwd = 2, col = "purple", lty=2)
legend("topleft",c("kernel density function estimates", "Observed BB Join count statistic"),
lty=c(1,2), col=c("red","purple"),cex=0.8)
legend("topright", legend = c("Permutation : 999",paste("p-value :",res.BB[[1]]$p.value)),cex=0.8)

# row-normalized rook matrix
res.BB<-joincount.mc(factor(as.vector(box1), labels=c("B","W")), nb2listw(nb.rook, style="W"), nsim=999)
dens.v <- density(res.BB[[1]]$res)
hist(res.BB[[1]]$res,col="lightblue",breaks=12,freq=F,xlab="Simulated BB join count statistic",
 main="Permutation test for BB join count statistic",ylim=c(0,1.1))
lines(dens.v, col="red",lwd=1.5)
segments(res.BB[[1]]$statistic,0, res.BB[[1]]$statistic, max(dens.v$y), lwd = 2, col = "purple", lty=2)
legend("topleft",c("kernel density function estimates", "Observed BB Join count statistic"),
lty=c(1,2), col=c("red","purple"),cex=0.8)
legend("topright", legend = c("Permutation : 999",paste("p-value :",res.BB[[1]]$p.value)),cex=0.8)


###################################
#####  Slide p.54
###################################

plot.M <- function(sp.obj, listw.obj, x, col.quad, label)
{# sp.obj: a Spatial object
 # listw.obj: a spatial weights object
 # x: a numeric vector
 # col.quad: a vector of 4 colors for the quadrants
 # label: the name of the variable
 
 # lag variable
 wx <- lag.listw(listw.obj, x)

 # quadrants
 x1 <- which((x>mean(x))&(wx>mean(wx)))
 x2 <- which((x>mean(x))&(wx<mean(wx)))
 x3 <- which((x<mean(x))&(wx<mean(wx)))
 x4 <- which((x<mean(x))&(wx>mean(wx)))

 # quadrant colors
 q.col <- rep("white", length(x))
 q.col[x1] <- col.quad[1]
 q.col[x2] <- col.quad[3]
 q.col[x3] <- col.quad[2]
 q.col[x4] <- col.quad[4]

 # local moran index
 resI <- localmoran(x, listw.obj)

 resI1 <- which((resI[,5]<0.05)&(x>mean(x))&(wx>mean(wx)))
 resI2 <- which((resI[,5]<0.05)&(x>mean(x))&(wx<mean(wx)))
 resI3 <- which((resI[,5]<0.05)&(x<mean(x))&(wx<mean(wx)))
 resI4 <- which((resI[,5]<0.05)&(x<mean(x))&(wx>mean(wx)))
 
 # plot
 layout(matrix(c(0,1,1,0,0,1,1,0,2,2,3,3,2,2,3,3), 4, 4, byrow = TRUE))
 # 1st graph
 res.moran<-moran.plot(x, listw.obj, main=paste("Moran plot of", label),
  xlab=label, ylab=paste("W x", label), col=q.col, pch=16, cex=0.5, labels=F)
 points(x[c(resI1, resI2, resI3, resI4)], wx[c(resI1, resI2, resI3, resI4)], 
  col=q.col[c(resI1, resI2, resI3, resI4)], pch=16, cex=1.25)
 legend("bottomright",c("H-H","H-L","L-L","L-H"),
  fill=col.quad[c(1,3,2,4)],title="Cluster",cex=0.8)
 legend("topleft", c("Influent point", "LISA sign.", "LISA non sign."),
  pch=c(9,16,16), pt.cex=c(1,1.4,0.5))
 
 # 2nd graph
 plot(sp.obj, col=add.alpha(q.col,0.3), border=add.alpha(q.col,0.3))
 which.inf <- which(apply(res.moran$is.inf,1,any))
 if(length(which.inf)>0)
  plot(sp.obj[which.inf,],col=q.col[which.inf], add=T, lwd=1)
 title("Influent points cluster map")
 legend("topleft",c("H-H","H-L","L-L","L-H"),fill=col.quad[c(1,3,2,4)],title="Cluster",cex=0.8)

 # 3rd graph
 plot(sp.obj, col=add.alpha(q.col,0.3), border=add.alpha(q.col,0.3))
 which.lisa <- c(resI1, resI2, resI3, resI4)
 if(length(which.lisa)>0)
 plot(sp.obj[which.lisa,],   col=q.col[which.lisa], add=T, lwd=1)
 title("Significant LISA cluster map")
 legend("topleft",c("H-H","H-L","L-L","L-H"),fill=col.quad[c(1,3,2,4)],title="Cluster",cex=0.8)
}

# applications :
plot.M(boston.tr, boston_listw, res.boston, rev(brewer.pal(4,"Spectral")), "residuals")  
plot.M(boston.tr, boston_listw, y, rev(brewer.pal(4,"Spectral")), "Y")

neg.autoco <- as.vector(invIrW(boston_listw, -0.9) %*% uncorr_x) 
plot.M(boston.tr, boston_listw, neg.autoco, rev(brewer.pal(4,"Spectral")), "Negative autocorrelated vector")  

# with GeoXp
 boston.tr$res <- residuals(boston.lm) 
 moranplotmap(boston.tr, "res", boston_listw)

